function [s,t,Sp_full,f,H,ph,fh]=tim_freq2time(Spectrum,Fstart,Fstop,F1,F2,F3,F4,interp_factor)
% [s,t,Sp_full,f,H,ph,fh]=tim_freq2time(Spectrum,Fstart,Fstop,F1,F2,F3,F4,interp_factor,[time_shift]) - function for converting the spectral data 
% into time domain.  
% The program applies a minimal-phase bandpass filter defined by frequencies F1-F4. 
% The frequencies must be Fstart <= F1 < F2 < F3 < F4 <= Fstop.
% F1 - the lower cut-off frequency, [Hz]
% F2 - the lower pass-band frequency (flat top), [Hz]
% F3 - the higher pass-band frequency (flat top), [Hz]
% F4 - the higher cut-off frequency, [Hz]
%
% If F1=F2=F3=F4=1, the program applies a minimal-phase Hanning window. 
% If F1=F2=F3=F4=0, the program doesn't apply any filtering or windowing.
%
% Output data:
% s  - signal in time domain;
% t  - vector of time, [s]
% Sp_full - full spectrum after pre-processing;
% f  - vector of frequencies, [Hz]
% H  - magnitude of window/filter;
% ph - phase of window/filter, [degrees];
% fh - frequencies of filter and input spectrum, [Hz]
%
% Input data:
% Spectrum - acquired complex spectrum (vector-column);
% Fstart   - start frequency, [Hz]
% Fstop    - stop frequency, [Hz] 
% interp_factor - interpolation factor  
%
% The program pre-process Spectrum, adds zeros from 0 Hz to Fstart, then
% adds zeros from Fstop to interp_factor*Fstop, creates the full spectrum
% and takes IFFT.

% T. Savelyev, IRCTR, TU Delft, 2007

[m,n]=size(Spectrum);
if m==1, Spectrum=Spectrum.'; end 
M=length(Spectrum);
dF=(Fstop-Fstart)/(M-1);
fh=[Fstart:dF:Fstop];

if (F1==1) & (F2==1) & (F3==1) & (F4==1),
    % minimal-phase windowing
    coeff=hanning(M);
    p1=imag(hilbert(-log(coeff + 1e-12)));
    Spectrum=Spectrum.*coeff.*exp(j*p1);
    H=coeff;
    ph=p1*180/pi;
elseif F4~=0
    % bandpass filtering
    Nf1=round((F1-Fstart)/dF);
    Nf2=round((F2-F1)/dF);
    Nf3=round((F4-F3)/dF);
    Nf4=round((Fstop-F4)/dF);  
    Nf5=M-Nf2-Nf1-Nf4-Nf3;
    coeff=hanning(2*Nf2+1);
    coeff1=coeff(1:Nf2);
    coeff=hanning(2*Nf3+1);
    coeff2=flipud(coeff(1:Nf3));
    coeff3=[coeff1; ones(Nf5,1); coeff2];
    p1=imag(hilbert(-log(coeff3 + 1e-12)));
    if Nf1>0,
        coeff3=[zeros(Nf1,1); coeff3];
        p1=[zeros(Nf1,1); p1];
    end
    if Nf4>0,
        coeff3=[coeff3; zeros(Nf4,1)];
        p1=[p1; zeros(Nf4,1)];
    end
    Spectrum=Spectrum.*coeff3.*exp(1j*p1);
    H=coeff3;
    ph=p1*180/pi;   
    
elseif (F1==0) & (F2==0) & (F3==0) & (F4==0),
    H=ones(M,1);
    ph=zeros(M,1);
end

% adding zeros for low frequencies up to Fstart
Nz=round(Fstart/dF);
Sp=[zeros(Nz,1); Spectrum];
N=length(Sp);
% adding zeros for higher frequencies (interpolation)
if interp_factor < 1, interp_factor=1; end
% total number of samples
Ns=interp_factor*N;
Sp1=[Sp; zeros(Ns-N,1)];
Sp2=conj(flipud(Sp1(2:end)));
Sp_full=[Sp1; 0; Sp2]/2;
Nfull=length(Sp_full);
% physical frequencies and time instants
Fs=dF*(Nfull-1);          % sampling frequency
f=[0:Nfull-1]'*dF;
dT=1/Fs;                  % sampling interval 
t=[(0:Nfull-1)*dT]';

% IFFT with normalizing 
s=real(ifft(Sp_full)); 



